/********************************/
/* Emmanuel Lochin              */
/* emmanuel.lochin@nicta.com.au */
/* TSW throughput estimator     */ 
/********************************/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <pcap.h>
#include <errno.h>
#include <signal.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <net/if.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <net/ethernet.h>
#include <math.h>

#include <unistd.h>
#include <sys/time.h>

#define SNAPLEN 100
#define AVG_INTERVAL 1   // 1 second


void
quit()
{
	printf("\n<Ctrl>+C recu, fermeture.\n");
	fclose(fp);
	exit(0);
}

void
flottant()
{
	printf("\nErreur flottant\n");
	exit(0);
}

void
trait_signaux(void)
{
	signal(SIGHUP, SIG_IGN);
	signal(SIGINT, quit);
	signal(SIGFPE, flottant);
	signal(SIGTERM, quit);
	signal(SIGKILL, quit);
	signal(SIGQUIT, quit);
}

int
main(int argc, char **argv)
{

	struct timeval  lasttime, curtime, lasttime2;
	struct timezone tz;
	u_long          begintime;

	float           avg_rate, Bytes_in_win = 0, New_bytes = 0;

	float           interval_us;
	float           interval_sec;
	long            len_pkt;
	double          Pktsize;
	char           *dev;

	char            errbuf[PCAP_ERRBUF_SIZE];
	pcap_t         *descr;
	const u_char * packet;
	struct pcap_pkthdr hdr;
	struct ether_header *eptr;
	struct ip      *iptr;
	struct tcphdr  *tcpptr;

	unsigned char  *adr_ip;
	u_int           dport;

	u_int           localnet, netmask;
	struct in_addr  inaddr;

	if (argc != 3) {
		printf("%s gives a throughput estimation (based on the Time Sliding Window algorithm) of a TCP flow.\n");
		printf("The measured flow is identified by its destination port.\n");
		printf("Syntaxe : %s <device> <dest-port>\n", argv[0]);
		printf("Exemple : ");
		printf("%s de0 7789\n", argv[0]);
		exit(EXIT_FAILURE);
	}
	dev = argv[1];
	dport = atoi(argv[2]);
	
	if ((descr = pcap_open_live(dev, SNAPLEN, 1, 1, errbuf)) == NULL) {
		printf("pcap_open_live(): %s\n", errbuf);
		return -1;
	}
	if (pcap_lookupnet(dev, &localnet, &netmask, errbuf) < 0)
		printf("%s\n", errbuf);
	inaddr.s_addr = localnet;
	inaddr.s_addr = netmask;

	/* Time initialisation */
	if (gettimeofday(&lasttime, &tz) != 0) {
		perror("gettimeofday");
		exit(EXIT_FAILURE);
	} else {
		//begintime = lasttime.tv_sec * 1000000L + lasttime.tv_usec;
		begintime = lasttime.tv_sec;
		//printf("Begintime %u sec \n\n", begintime);
	}

	if (gettimeofday(&lasttime2, &tz) != 0) {
		perror("gettimeofday");
		exit(EXIT_FAILURE);
	}
	/* End Time initialisation */

	/* next packet */

	trait_signaux();
				   
	while (1) {

		if ((packet = pcap_next(descr, &hdr)) != NULL) {
			eptr = (struct ether_header *) packet;
			iptr = (struct ip *) (14 + packet);
			tcpptr = (struct tcphdr *) (14 + 20 + packet);

			/* Type of pkt */

			if (ntohs(eptr->ether_type) == ETHERTYPE_IP) {
				if (gettimeofday(&curtime, &tz) != 0) {
					perror("gettimeofday");
					exit(EXIT_FAILURE);
				}
				adr_ip = (unsigned char *)(&iptr->ip_src);

				if ((int)ntohs(tcpptr->th_dport) == dport) {
					interval_us = ((curtime.tv_sec * 1000000L + curtime.tv_usec) \
						       -(lasttime.tv_sec * 1000000L + lasttime.tv_usec));
					interval_sec = (float)interval_us / 1000000;

					len_pkt = ntohs(iptr->ip_len);
					
					Pktsize = (8 * (float)len_pkt) / (1024 * 1024);

					Bytes_in_win = avg_rate * AVG_INTERVAL;
					New_bytes = Bytes_in_win + Pktsize;
					avg_rate = New_bytes / (interval_sec + AVG_INTERVAL);

					lasttime = curtime;
				} 


			if ((curtime.tv_sec * 1000000L + curtime.tv_usec) \
				    -(lasttime2.tv_sec * 1000000L + lasttime2.tv_usec) >= 1000000) {
					printf("avg_rate %.0fKbit/s <=> %.3fMbit/s \n", 1000*avg_rate, avg_rate);

					lasttime2 = curtime;
			}
				

			} 
		} 
	} 
	return 0;
}
